@@ -16,15 +16,28 @@ pub struct Requirement {
     extras: Vec<String>,
     version_or_url: Option<VersionOrUrl>,
     marker: Option<MarkerExpr>,
+    private: bool,
 }
 
 impl Requirement {
     pub fn new(raw: &str) -> Result<Self, String> {
-        // Extract the marker (find ';' separator)
-        let (raw_req, marker_start) = if let Some(idx) = raw.find(';') {
-            (&raw[..idx], Some(idx + 1))
+        // Check for ; private suffix (PEP 794)
+        let (raw_without_private, private) = if let Some(idx) = raw.rfind(';') {
+            let suffix = raw[idx + 1..].trim();
+            if suffix.eq_ignore_ascii_case("private") {
+                (&raw[..idx], true)
+            } else {
+                (raw, false)
+            }
         } else {
-            (raw, None)
+            (raw, false)
+        };
+
+        // Extract the marker (find ';' separator)
+        let (raw_req, marker_start) = if let Some(idx) = raw_without_private.find(';') {
+            (&raw_without_private[..idx], Some(idx + 1))
+        } else {
+            (raw_without_private, None)
         };
 
         // Extract URL from remaining (find "@" separator)
@@ -45,7 +58,7 @@ impl Requirement {
             (name, extras)
         } else {
             // No extras, but need to find where version specifiers start
-            let name_end = req_part.find(|c: char| "=!<>~".contains(c)).unwrap_or(req_part.len());
+            let name_end = req_part.find(|c: char| "=!<>~(".contains(c)).unwrap_or(req_part.len());
             let name = &req_part[..name_end];
             (name, vec![])
         };
@@ -55,7 +68,8 @@ impl Requirement {
             let url = &raw_req[url_idx..];
             Some(VersionOrUrl::Url(url.trim().to_string()))
         } else if let Some(specs_start) = req_part.find(|c: char| "=!<>~".contains(c)) {
-            let specs = &req_part[specs_start..];
+            let specs_end = req_part.find(|c: char| ")".contains(c)).unwrap_or(req_part.len());
+            let specs = &req_part[specs_start..specs_end];
             let mut parsed = Vec::new();
             for spec in specs.split(',') {
                 match VersionOp::new(spec) {
@@ -69,7 +83,7 @@ impl Requirement {
         };
 
         let marker = if let Some(marker_idx) = marker_start {
-            let marker_str = raw[marker_idx..].trim();
+            let marker_str = raw_without_private[marker_idx..].trim();
             if marker_str.is_empty() {
                 None
             } else {
@@ -83,6 +97,7 @@ impl Requirement {
             extras,
             version_or_url,
             marker,
+            private,
         })
     }
 
@@ -159,6 +174,9 @@ impl std::fmt::Display for Requirement {
         if let Some(marker) = &self.marker {
             write!(&mut result, "; {marker}")?;
         }
+        if self.private {
+            write!(&mut result, "; private")?;
+        }
         write!(f, "{}", result)
     }
 }
