@@ -4,7 +4,7 @@ use taplo::formatter::{format_syntax, Options};
 use taplo::parser::parse;
 use taplo::syntax::SyntaxKind::{ENTRY, VALUE};
 
-use crate::array::{sort_strings, transform};
+use crate::array::{dedupe_strings, sort, sort_strings, transform};
 use crate::pep508::Requirement;
 
 #[rstest]
@@ -170,6 +170,14 @@ fn test_order_array(#[case] start: &str, #[case] expected: &str) {
         indoc ! {r#"a=["B","A"]"#},
         indoc ! {r#"a=["A","B"]"#}
 )]
+#[case::single_element_no_comma(
+        indoc ! {r#"a=["A"]"#},
+        indoc ! {r#"a=["A"]"#}
+)]
+#[case::empty_array(
+        indoc ! {r#"a=[]"#},
+        indoc ! {r#"a=[]"#}
+)]
 fn test_reorder_no_trailing_comma(#[case] start: &str, #[case] expected: &str) {
     let root_ast = parse(start).into_syntax().clone_for_update();
     for children in root_ast.children_with_tokens() {
@@ -187,3 +195,84 @@ fn test_reorder_no_trailing_comma(#[case] start: &str, #[case] expected: &str) {
     res.retain(|x| !x.is_whitespace());
     assert_eq!(res, expected);
 }
+
+#[test]
+fn test_sort_empty_array_direct() {
+    let start = r#"a=[]"#;
+    let root_ast = parse(start).into_syntax().clone_for_update();
+    for children in root_ast.children_with_tokens() {
+        if children.kind() == ENTRY {
+            for entry in children.as_node().unwrap().children_with_tokens() {
+                if entry.kind() == VALUE {
+                    sort::<String, _, _>(entry.as_node().unwrap(), |_| None, &|lhs, rhs| lhs.cmp(rhs));
+                }
+            }
+        }
+    }
+    let res = root_ast.to_string();
+    assert_eq!(res, "a=[]");
+}
+
+#[rstest]
+#[case::no_duplicates(
+        indoc ! {r#"
+    a = ["A", "B", "C"]
+    "#},
+        indoc ! {r#"
+    a = ["A", "B", "C"]
+    "#}
+)]
+#[case::basic_duplicates(
+        indoc ! {r#"
+    a = ["A", "a", "B", "A"]
+    "#},
+        indoc ! {r#"
+    a = ["A", "B"]
+    "#}
+)]
+#[case::empty_array(
+        indoc ! {r"
+    a = []
+    "},
+        indoc ! {r"
+    a = []
+    "}
+)]
+#[case::multiline_with_trailing_duplicate(
+        indoc ! {r#"
+    a = [
+      "A",
+      "B",
+      "a",
+    ]
+    "#},
+        indoc ! {r#"
+    a = ["A", "B"]
+    "#}
+)]
+#[case::duplicate_at_end_no_trailing_comma(
+        indoc ! {r#"
+    a = ["A", "B", "a"]
+    "#},
+        indoc ! {r#"
+    a = ["A", "B"]
+    "#}
+)]
+fn test_dedupe_strings(#[case] start: &str, #[case] expected: &str) {
+    let root_ast = parse(start).into_syntax().clone_for_update();
+    for children in root_ast.children_with_tokens() {
+        if children.kind() == ENTRY {
+            for entry in children.as_node().unwrap().children_with_tokens() {
+                if entry.kind() == VALUE {
+                    dedupe_strings(entry.as_node().unwrap(), |s| s.to_lowercase());
+                }
+            }
+        }
+    }
+    let opt = Options {
+        column_width: 120,
+        ..Options::default()
+    };
+    let res = format_syntax(root_ast, opt);
+    assert_eq!(res, expected);
+}
